@@ -60,12 +60,14 @@ def get_limits(vmin=None, vmax=None, datavals=None, lognorm=False, i=0):
     if lognorm:
         if (v0<=0): v0 = 1e-2
         if (v1<=0): v1 = 1.
-        tick0 = np.ceil(np.log10(v0))
+        tick0 = np.floor(np.log10(v0))
         tick1 = np.ceil(np.log10(v1))
         pow = np.arange(tick0, tick1, 1)
         ticks=[]
         for j in pow:
-            ticks.extend([10**j, 2*10**j, 5*10**j])
+            for tscale in [1, 2, 5]:
+                if v0 <= tscale*10**j and v1 >= tscale*10**j:
+                    ticks.extend([tscale*10**j])
         ticklab = ["%g" % val for val in ticks]
     else:
         dex = 10**np.floor(np.log10(abs(v1-v0)))
@@ -89,7 +91,7 @@ def get_limits(vmin=None, vmax=None, datavals=None, lognorm=False, i=0):
 
 def props_colmap(dendrogram=None, subcat=None, img=None, cubhd=None, 
         props=['tmax'], vmin=None, vmax=None, lognorm=False,
-        cmapname='jet', prefix='output', **kwargs):
+        cmapname='jet', prefix='output', noshow=False, **kwargs):
     print("Image leaves and clusters colored by properties")
     srclist = subcat['_idx'].tolist()
     # Make a plot for each requested property
@@ -108,6 +110,10 @@ def props_colmap(dendrogram=None, subcat=None, img=None, cubhd=None,
         datavals=subcat[type]
         if lognorm:
             datavals[datavals<=0] = np.nan
+        if (~np.isfinite(datavals)).all():
+            print('All data for {} are NaN'.format(type))
+            plt.close()
+            continue
         plt.tick_params(axis='both', which='both', bottom=False, top=False, 
             left=False, labelleft=False, labeltop=False)
         v0, v1, ticks, tlbl = get_limits(vmin=vmin, vmax=vmax, datavals=datavals, 
@@ -136,7 +142,8 @@ def props_colmap(dendrogram=None, subcat=None, img=None, cubhd=None,
             cbar.set_label(name+' ['+str(subcat[type].unit)+']',size=12,labelpad=10)
         plt.savefig(prefix+'_'+type.replace("_","")+'.pdf', 
             bbox_inches='tight')
-        plt.close()
+        if noshow:
+            plt.close(fig)
     return
 
 #%&%&%&%&%&%&%&%&%&%&%&%&%&%%&%&%&%&%&%&%&%&%
@@ -145,7 +152,7 @@ def props_colmap(dendrogram=None, subcat=None, img=None, cubhd=None,
 
 def props_coltree(label=None, dendrogram=None, cat=None, cubhd=None, 
         props=['tmax'], cmapname='jet', vmin=None, vmax=None, lognorm=False,
-        xmin=None, xmax=None, prefix='output'):
+        xmin=None, xmax=None, prefix='output', noshow=False):
     print("Draw tree diagram colored by properties")
     # Make a plot for each requested property
     for i, type in enumerate(props):
@@ -157,7 +164,14 @@ def props_coltree(label=None, dendrogram=None, cat=None, cubhd=None,
             ax.set_xlim(xmin,xmax)
         p = dendrogram.plotter()
         name = scale_values(cat=cat, type=type, cubhd=cubhd)
-        v0, v1, ticks, tlbl = get_limits(vmin=vmin, vmax=vmax, datavals=cat[type],
+        datavals = cat[type]
+        if lognorm:
+            datavals[datavals<=0] = np.nan
+        if (~np.isfinite(datavals)).all():
+            print('All data for {} are NaN'.format(type))
+            plt.close()
+            continue
+        v0, v1, ticks, tlbl = get_limits(vmin=vmin, vmax=vmax, datavals=datavals,
                                         lognorm=lognorm, i=i)
         print('{} vmin and vmax: {} {}'.format(type,v0,v1))
         cmap = plt.cm.get_cmap(cmapname)
@@ -188,14 +202,15 @@ def props_coltree(label=None, dendrogram=None, cat=None, cubhd=None,
                 alpha=0.7), xycoords='data', textcoords='offset points')
         plt.savefig(prefix+'_dendrogram_'+type.replace("_","")+'.pdf', 
             bbox_inches='tight')
-        plt.close()
+        if noshow:
+            plt.close(fig)
     return
 
 # -------------------------------------------------------------------------------
 
 def colorcode(label='scimes', table='full_catalog', cubefile=None, mom0file=None, 
-        types=['v_cen','v_rms','tmax','imean'], outdir='plots', vmin=None, vmax=None,
-        lognorm=False, **kwargs):
+        types=['v_cen','v_rms','tmax','imean'], plotdir='plots', vmin=None, vmax=None,
+        lognorm=False, noshow=False, **kwargs):
     # Header info
     hdu3 = fits.open(cubefile)[0]
     hd3 = hdu3.header
@@ -204,10 +219,11 @@ def colorcode(label='scimes', table='full_catalog', cubefile=None, mom0file=None
     img = hdu2.data
     # Load the dendrogram
     d = Dendrogram.load_from(label+'_dendrogram.hdf5')
-    print('\n')
+    print('\nOpening '+label+'_'+table+'.txt'+'\n')
     cat = Table.read(label+'_'+table+'.txt', format='ascii.ecsv')
     # Plot colored ellipses on maps
     for set in ['leaves', 'trunks', 'clusters']:
+        print('Working on', set)
         try:
             idc = []
             with open(label+'_'+set+'.txt', 'r') as f:
@@ -216,11 +232,11 @@ def colorcode(label='scimes', table='full_catalog', cubefile=None, mom0file=None
                     idc.append(int(row[0]))
             subcat = cat[idc]
             props_colmap(dendrogram=d, subcat=subcat, img=img, cubhd=hd3,
-                props=types, prefix=outdir+'/'+label+'_'+set, vmin=vmin, vmax=vmax, 
-                lognorm=lognorm, **kwargs)
+                props=types, prefix=plotdir+'/'+label+'_'+set, vmin=vmin, vmax=vmax, 
+                lognorm=lognorm, noshow=noshow, **kwargs)
         except IOError:
             print(label,set,'not found')
     # Plot colored dendrogram
     props_coltree(label=label, dendrogram=d, cat=cat, cubhd=hd3, props=types, 
-        prefix=outdir+'/'+label, vmin=vmin, vmax=vmax, lognorm=lognorm)
+        prefix=plotdir+'/'+label, vmin=vmin, vmax=vmax, lognorm=lognorm, noshow=noshow)
     return
