@@ -2,8 +2,7 @@
 
 import numpy as np
 from numpy.random import randint
-from scipy import stats
-from scipy import odr
+from scipy import stats, odr
 import os
 from os.path import join
 import re
@@ -108,7 +107,7 @@ def sctplot(xdata, ydata, xerr=None, yerr=None, zdata=None, cmap=None,
 
 def color_code_bin(x, y, z, xerr=None, yerr=None, lobin=25, lobin_col='cyan', 
             hibin=75, hibin_col='salmon', ms=6, xlims=None, nbin=8, binms=8,
-            colname=None, alpha=None, cmap='jet', axes=None, zlog=False):
+            colname=None, alpha=None, cmap='jet', axes=None, zlog=False, median=False):
     '''
     Color coded scatter plot with binned values in upper and lower quartiles.
 
@@ -149,6 +148,8 @@ def color_code_bin(x, y, z, xerr=None, yerr=None, lobin=25, lobin_col='cyan',
         Plot axes object
     zlog : boolean
         True if z-values are logarithmic, used for labeling colorbar only.
+    median : boolean
+        True to also plot binned medians as gray steps
     '''
     if axes is None:
         axes = plt.gca()
@@ -168,36 +169,55 @@ def color_code_bin(x, y, z, xerr=None, yerr=None, lobin=25, lobin_col='cyan',
     if nbin > 0:
         if xlims is None:
             xlims = axes.get_xlim()
+        ylocnt, xbinedge, _ = stats.binned_statistic(x[z < q1], y[z < q1],
+            statistic='count', bins=nbin, range=xlims)
         ylomean, xbinedge, _ = stats.binned_statistic(x[z < q1], y[z < q1],
             statistic='mean', bins=nbin, range=xlims)
         ylostd, xbinedge, _  = stats.binned_statistic(x[z < q1], y[z < q1],
             statistic='std', bins=nbin, range=xlims)
         xbin = 0.5*(xbinedge[1:]+xbinedge[:-1])
-        axes.errorbar(xbin, ylomean, yerr=ylostd, ecolor='k', marker='o', 
+        axes.errorbar(xbin[ylocnt>1], ylomean[ylocnt>1], yerr=ylostd[ylocnt>1], 
+                    ecolor='k', marker='o', 
                     ms=binms, ls='', mfc=lobin_col, mec='k', zorder=3)
+        if len(xbin[ylocnt>1]) > 2:
+            b, a, rval, pval, std_err = stats.linregress(xbin[ylocnt>1], ylomean[ylocnt>1])
+            print("Lower quartile fit: {:.2f} + x*({:.2f}+/-{:.2f})".format(
+                   a, b, std_err))
+            #plt.plot(xbin[ylocnt>1], a+b*xbin[ylocnt>1], 'r-', linewidth=3)
+        yhicnt, xbinedge, _ = stats.binned_statistic(x[z > q2], y[z > q2],
+            statistic='count', bins=nbin, range=xlims)
         yhimean, xbinedge, _ = stats.binned_statistic(x[z > q2], y[z > q2],
             statistic='mean', bins=nbin, range=xlims)
         yhistd, xbinedge, _  = stats.binned_statistic(x[z > q2], y[z > q2],
             statistic='std', bins=nbin, range=xlims)
-        axes.errorbar(xbin, yhimean, yerr=yhistd, ecolor='k', marker='o', 
+        axes.errorbar(xbin[yhicnt>1], yhimean[yhicnt>1], yerr=yhistd[yhicnt>1], 
+                    ecolor='k', marker='o', 
                     ms=binms, ls='', mfc=hibin_col, mec='k', zorder=3)
+        if len(xbin[yhicnt>1]) > 2:
+            b, a, rval, pval, std_err = stats.linregress(xbin[yhicnt>1], yhimean[yhicnt>1])
+            print("Upper quartile fit: {:.2f} + x*({:.2f}+/-{:.2f})".format(
+                   a, b, std_err))
+            #plt.plot(xbin[yhicnt>1], a+b*xbin[yhicnt>1], 'b-', linewidth=3)
+        if median:
+            ymedian, xbinedge, _ = stats.binned_statistic(x, y,
+                statistic='median', bins=nbin, range=xlims)
+            axes.step(xbinedge , np.append(ymedian, ymedian[-1]), where='post', ls='-', 
+                      color='grey', linewidth=4, alpha=0.5, zorder=3)
         # Colorbar annotations
-        axis_to_data = cb.ax.transAxes + cb.ax.transData.inverted()
-        data_to_axis = axis_to_data.inverted()
-        #cb.ax.axhline( y=q1, c=lobin_col, linewidth=4 )
-        dnarrow = data_to_axis.transform([q1,q1])[0]
-        cb.ax.errorbar( 0.5, dnarrow, yerr=0.03, color=lobin_col, uplims=True,
-                         elinewidth=2, ecolor='k', marker='o', mec='k', ms=binms,
+        dnarrow = cb.ax.transLimits.transform([q1,q1])
+        print('dnarrow:', dnarrow)
+        cb.ax.errorbar( 0.5, dnarrow[1], yerr=0.04, color=lobin_col, uplims=True,
+                         elinewidth=2, ecolor='grey', marker='o', mec='k', ms=binms,
                          transform=cb.ax.transAxes)
-        #cb.ax.axhline( y=q2, c=hibin_col, linewidth=4 )
-        uparrow = data_to_axis.transform([q2,q2])[0]
-        cb.ax.errorbar( 0.5, uparrow, yerr=0.03, color=hibin_col, lolims=True,
-                         elinewidth=2, ecolor='k', marker='o', mec='k', ms=binms,
+        uparrow = cb.ax.transLimits.transform([q2,q2])
+        print('uparrow:', uparrow)
+        cb.ax.errorbar( 0.5, uparrow[1], yerr=0.04, color=hibin_col, lolims=True,
+                         elinewidth=2, ecolor='grey', marker='o', mec='k', ms=binms,
                          transform=cb.ax.transAxes)
     return
 
 
-def std_overlay(cat, axvar, xlims=None, ylims=None, shade=None, axes=None):
+def std_overlay(cat, axvar, xlims=None, ylims=None, shade=None, axes=None, panel=None):
     '''
     Generate axis labels and custom overlays on log-log plots
 
@@ -228,7 +248,8 @@ def std_overlay(cat, axvar, xlims=None, ylims=None, shade=None, axes=None):
         'siglum':'$\Sigma$, CO-based',
         'siglte':'$\Sigma$, LTE-based',
         'sigvir':'$\Sigma$, virial',
-        'rad_pc':'spherical radius',
+        'rad_pc':'equivalent raw radius',
+        'rad_pc_dcon':'equivalent radius',
         'vrms_k':'rms linewidth',
         'area_pc2':'projected area',
        }
@@ -253,12 +274,12 @@ def std_overlay(cat, axvar, xlims=None, ylims=None, shade=None, axes=None):
             axes.axhspan(ylims[0], np.log10(shade[axvar[1]]), fc='lightgray', 
                          alpha=0.3, lw=0)
     # Solomon et al. size-linewidth relation
-    if axvar[0] == 'rad_pc' and axvar[1] == 'vrms_k':
+    if axvar[0].startswith('rad_') and axvar[1] == 'vrms_k':
         xmod = np.linspace(xlims[0],xlims[1],20)
         ymod = np.log10(0.72) + 0.5*xmod
         axes.plot(xmod, ymod, linestyle='-', color='r', lw=4, alpha=0.5, 
             zorder=-1)
-        axes.text((xlims[1]-0.05), (xlims[1]/2-0.15), 'S87', 
+        axes.text((xlims[1]-0.05), (xlims[1]/2-0.23), 'S87', 
             horizontalalignment='right', color='r', rotation=25)
     # Lines of constant surface density and volume density
     if axvar[0] == 'rad_pc' and axvar[1].startswith('m'):
@@ -289,10 +310,12 @@ def std_overlay(cat, axvar, xlims=None, ylims=None, shade=None, axes=None):
     if axvar[0].startswith('sig') and axvar[1] == 'sigvir':
         xmod = np.linspace(xlims[0],xlims[1],100)
         ymod = np.log10(10**xmod + (20/(3*np.pi*21.1))*1.e4/10**xmod)
+        ymod6 = np.log10(10**xmod + (20/(3*np.pi*21.1))*1.e6/10**xmod)
+        axes.plot(xmod, ymod6, linestyle='-.', color='brown', lw=1)
         axes.plot(xmod, ymod, linestyle='-', color='g', lw=1)
-        ymod2 = np.log10(10**xmod + (20/(3*np.pi*21.1))*1.e2/10**xmod)
-        axes.plot(xmod, ymod2, linestyle=':', color='m', lw=1)
         if xlims[0] == -1:
+            ymod2 = np.log10(10**xmod + (20/(3*np.pi*21.1))*1.e2/10**xmod)
+            axes.plot(xmod, ymod2, linestyle=':', color='m', lw=1)
             axes.text(-0.9, 2.40, '$P_{ext}$ = $10^4$ cm$^{-3}$ K', ha='left',
                 color='g', rotation=-45)
             axes.text(-0.9, 0.50, '$P_{ext}$ = $10^2$ cm$^{-3}$ K', ha='left',
@@ -306,6 +329,11 @@ def std_overlay(cat, axvar, xlims=None, ylims=None, shade=None, axes=None):
         else:
             ymod = xmod
         axes.plot(xmod, ymod, linestyle=':', color='k')
+        if axvar[0].startswith('sig') and axvar[1] == 'sigvir':
+            axes.text(0.05, 0.1, r'$\alpha$ > 1', ha='left',
+                color='k', rotation=45, transform=axes.transAxes)            
+            axes.text(0.1, 0.05, r'$\alpha$ < 1', ha='left',
+                color='k', rotation=45, transform=axes.transAxes)            
     # Set plot limits and labels
     axes.set_xlim(xlims[0], xlims[1])
     axes.set_ylim(ylims[0], ylims[1])
@@ -462,15 +490,14 @@ def linefitting(x, y, xerr=None, yerr=None, color='b', prob=.95,
 # -------------------------------------------------------------------------------
 
 def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2, 
-            alpha=1, cmap='jet', doline=True, 
+            alpha=1, cmap='jet', parprint=False,
             nbin=0, lobin_col='cyan', hibin_col='salmon',
             xplot=['rad_pc',   'vrms_k','area_pc2'],
             yplot=['vrms_k',   'mlumco',  'mlumco'],
-            xlims=[[-1.5,1],     [-2,2],    [-1,3]],
-            ylims=[[-2,1.5], [-1.5,4.5],    [-2,4]],
             pltname=[ 'rdv',   'dvflux','areaflux'],
-            ccode=[    True,      False,     False], 
-            colorcodes=['alpha', 'siglum', 'sigvir']):
+            xlims=[], ylims=[], doline=[], panel=[], ccode=[],
+            physnam='physprop', resolve_output='none', 
+            colorcodes=['alpha', 'sigvir'], noshow=False):
     '''
     Generate a set of summary plots for an astrodendro run
 
@@ -491,9 +518,8 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
         Transparency parameter for color-coded scatter plots, between 0 and 1.
     cmap : matplotlib.colors.Colormap
         Name of the color map for color coding
-    doline : boolean
-        True to plot best-fit line and confidence band for 'full' plots.  
-        Otherwise the parameters are calculated but not plotted.
+    parprint : boolean
+        True to show best-fit slope and intercept in upper left corner of plot
     nbin : int
         Number of binned averages to generate across horizontal axis of each plot.
         Default is not to perform binning (nbin=0).
@@ -504,20 +530,34 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
     xplot : list of str
         Columns to plot on x axis, should be in 'physprop' catalog
     yplot : list of str
-        Column sto plot on y axis, should be in 'physprop' catalog
-    xlims : list of tuple
-        The x limits of the desired plots
-    ylims : list of tuple
-        The y limits of the desired plots
+        Columns to plot on y axis, should be in 'physprop' catalog
     pltname : list of str
         File name identifiers for the desired plots
+    xlims : list of tuple
+        The x limits of the desired plots.  List should match length of xplots.
+        Default: autoscale
+    ylims : list of tuple
+        The y limits of the desired plots.  List should match length of xplots.
+        Default: autoscale
+    doline : list of boolean
+        True to plot best-fit line and confidence band for full & cluster plots.
+        Default: parameters are calculated but not plotted.
     ccode : list of boolean
-        Whether to generate the color coded versions of each plot
-    colorcodes :
+        Whether to generate the color coded versions of each plot.  If given, this
+        should be an array with the same length as xplot and yplot.
+        Default: do not generate these
+    colorcodes : list of str
         Columns to plot as color codes, these can be either in 'full' catalog
         or 'physprop' catalog.
-        
+    resolve_output: string
+        Set to true to output the well-resolved structures to a new output file.
+        'raw'  : choose based on all resolution limits
+        'decon': choose based on valid deconvolved size only
+        'none' : do not output
+        Default is 'none'.
     '''
+
+    # ------ Get the resolution limits
     deltav  = dvkms  * u.km / u.s
     avgbeam = beam   * u.arcsec
     dist    = distpc * u.pc
@@ -539,15 +579,15 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
         except OSError:
             print('Unable to create output directory',plotdir)
 
-    # Read the input files
+    # ------ Read the input files
     cat = Table.read(catalog, format='ascii.ecsv')
-    pcatalog = catalog.replace('_full_catalog.txt','_physprop_add.txt')
     label = (os.path.basename(catalog)).replace('_full_catalog.txt','')
     indir = os.path.dirname(catalog)
+    pcatalog = catalog.replace('_full_catalog.txt','_'+physnam+'_add.txt')
     if os.path.isfile(pcatalog):
         pcat = Table.read(pcatalog, format='ascii.ecsv')
     else:
-        pcatalog = catalog.replace('_full_catalog.txt','_physprop.txt')
+        pcatalog = catalog.replace('_full_catalog.txt','_'+physnam+'.txt')
         pcat = Table.read(pcatalog, format='ascii.ecsv')
 
     # Get the indices of trunks, branches, leaves, and clusters.
@@ -572,6 +612,7 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
             for line in text.splitlines():
                 trd.append(list(map(int, line.split('|')[1].split(','))))
     except:
+        print('Failed reading',join(indir,label+'_trunks.txt'))
         pass
     
     # Get the lists of cluster descendants and colors
@@ -581,13 +622,17 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
             cld = []
             clco = []
             for line in text.splitlines():
-                cld.append(list(map(int, line.split('|')[1].split(','))))
-                clco.append(line.split()[1]) 
+                clco.append(line.split()[1])
+                if line.split('|')[1].split(',') == [' ']:
+                    cld.append([])
+                else:
+                    cld.append(list(map(int, line.split('|')[1].split(','))))
     except:
+        print('Failed reading',join(indir,label+'_clusters.txt'))
         pass
 
-    # Plot histograms of selected properties.  First plot is position angle
-    # and is drawn linearly, subsequent plots are logarithmic.
+    # ------ Plot histograms of selected properties.  No resolution filtering!
+    # First plot is PA and is drawn linearly, subsequent plots are logarithmic.
     hist_struct = ['trunks', 'branches', 'leaves']
     hist_plots  = ['position_angle', 'flux', 'mvir']
     for i, histcol in enumerate(hist_plots):
@@ -627,52 +672,24 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
         axes.set_ylabel('Number')
         plt.legend(loc='best',fontsize='medium')
         plt.savefig(join(plotdir,label+'_'+histcol+'.pdf'), bbox_inches='tight')
-        plt.close()
-
-    # Plot all structures, color coded by 3rd variable and binned in quartiles
-    if isinstance(colorcodes, str):
-        colorcodes = [colorcodes]
-    for i in range(len(xplot)):
-        if not ccode[i]:
-            continue
-        print('\nPlotting',xplot[i],'and',yplot[i])
-        x, y = [pcat[xplot[i]], pcat[yplot[i]]]
-        postive = (x>0) & (y>0)
-        for j in range(len(colorcodes)):
-            fig, axes = plt.subplots(figsize=(6.4,4.8))
-            axes.set_aspect('equal')
-            print('Relation will be color coded by',colorcodes[j])
-            if colorcodes[j] not in cat.keys() and colorcodes[j] not in pcat.keys():
-                continue
-            if colorcodes[j] in cat.keys():
-                zcode = cat[colorcodes[j]][postive]
-                zlog = False
-            elif colorcodes[j] in ['axratio', 'refdist']:
-                zcode = pcat[colorcodes[j]][postive]
-                zlog = False
-            else:
-                zcode = np.log10(pcat[colorcodes[j]][postive])
-                zlog = True
-            color_code_bin(np.log10(x[postive]), np.log10(y[postive]), zcode, 
-                           colname=colorcodes[j], alpha=alpha, cmap=cmap, axes=axes,
-                           lobin_col=lobin_col, hibin_col=hibin_col, xlims=xlims[i], 
-                           zlog=zlog, nbin=nbin)
-            std_overlay(pcat, [xplot[i], yplot[i]], xlims[i], ylims[i], shade)
-            shortname = re.sub('_', '', colorcodes[j])
-            plt.savefig(join(plotdir,label+'_'+pltname[i]+'_'+shortname+'.pdf'), 
-                        bbox_inches='tight')
+        if noshow:
             plt.close()
 
-    # Output table with line fitting parameters
-    tab = Table(dtype=[('cloud', 'S10'), ('pltname', 'S10'), ('a', 'f4'), 
-                    ('a_err', 'f4'), ('b', 'f4'), ('b_err', 'f4'), 
+    # ------ Get color code requests
+    if isinstance(colorcodes, str):
+        colorcodes = [colorcodes]
+
+    # ------ Output table with line fitting parameters
+    tab = Table(dtype=[('cloud', 'S10'), ('pltname', 'S10'), ('npts', 'i4'), 
+                    ('a', 'f4'), ('a_err', 'f4'), ('b', 'f4'), ('b_err', 'f4'), 
                     ('chi2red', 'f4'), ('eps', 'f4')])
     for col in ['a', 'a_err', 'b', 'b_err', 'chi2red', 'eps']:
         tab[col].format = '.2f'
 
-    # Main set of scatter plots, as requested by user
+    # ------ Main set of scatter plots, as requested by user
     for i in range(len(xplot)):
-        print('\nPlotting',xplot[i],'and',yplot[i])
+
+        print('\n*** Plotting',xplot[i],'and',yplot[i])
         x, y = [pcat[xplot[i]], pcat[yplot[i]]]
         if 'e_'+xplot[i] in pcat.keys():
             xerr = pcat['e_'+xplot[i]]
@@ -682,23 +699,25 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
             yerr = pcat['e_'+yplot[i]]
         else:
             yerr = y*0 + 0.1
-        # --- Must be positive to take logarithm
-        postive = (x>0) & (y>0) & (xerr>0) & (yerr>0)
-        # --- Restrict indices of subsets to positive values
-        idsel = idc[:]
-        for j in range(4):
-            idsel[j] = [val for val in idc[j] if val in np.where(postive)[0]]
-        # --- Exclude unresolved points from line fitting
-        xmin = ymin = 0
+
+        # --- Select points for display: default is all positive values
+        dodisp = (x>0) & (y>0) & (xerr>0) & (yerr>0)
+        print('{} points selected for display'.format(np.count_nonzero(dodisp)))
+        # --- Display unresolved points (dodisp) but exclude from line fitting (dofit)
+        dofit = dodisp
         if xplot[i] in shade.keys():
             print('Excluding points from {} below {}'.format(xplot[i],shade[xplot[i]]))
-            if shade[xplot[i]] > 0:
-                xmin = shade[xplot[i]]
+            dofit = dofit & (x > shade[xplot[i]])
         if yplot[i] in shade.keys():
             print('Excluding points from {} below {}'.format(yplot[i],shade[yplot[i]]))
-            if shade[yplot[i]] > 0:
-                ymin = shade[yplot[i]]
-        unshade = (x>xmin) & (y>ymin) & (xerr>0) & (yerr>0)
+            dofit = dofit & (y > shade[yplot[i]])
+            
+        # --- Restrict indices of subsets to positive or resolved values
+        idsel = idc[:]
+        idfit = idc[:]
+        for j in range(4):
+            idsel[j] = [val for val in idc[j] if val in np.where(dodisp)[0]]
+            idfit[j] = [val for val in idc[j] if val in np.where(dofit)[0]]
 
         # --- Plot trunks, branches, and leaves together
         fig, axes = plt.subplots(figsize=(6.4,4.8))
@@ -707,7 +726,7 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
         else:
             axes.set_aspect('equal')
         # Get legend label
-        reg = label.split('_')[0]
+        reg  = label.split('_')[0]
         line = label.split('_')[-1]
         if line == '12':
             plt.plot([], [], ' ', label=reg+' CO')
@@ -725,68 +744,167 @@ def pltprops(catalog, plotdir='plots', distpc=5e4, dvkms=0.2, beam=2,
         sctplot( np.log10(x[idsel[2]]), np.log10(y[idsel[2]]), 
                  xerr=xerr[idsel[2]]/np.log(10), yerr=yerr[idsel[2]]/np.log(10), 
                  col='green', marker='o', mec='k', ms=15, zorder=3, label='leaves' )
+
+        # Set the axis limits
+        try:
+            this_xlims = xlims[i]
+            this_ylims = ylims[i]
+        except IndexError:
+            this_xlims = axes.get_xlim()
+            this_ylims = axes.get_ylim()
+        # Decide whether to plot a line
+        try:
+            drawline = doline[i]
+        except IndexError:
+            drawline = False
+        # Decide whether to label as subpabel
+        try:
+            subpanel = panel[i]
+        except IndexError:
+            subpanel = None
+        # Decide whether to generate the colorcode plots
+        try:
+            this_ccode = ccode[i]
+        except IndexError:
+            this_ccode = False
+        
         # Plot the best-fitting line and confidence interval
-        if len(x[unshade]) > 2:
-            a1, a1_e, a0, a0_e, chi2, eps = linefitting( np.log10(x[unshade]), 
-                np.log10(y[unshade]), xerr=xerr[unshade]/np.log(10), 
-                yerr=yerr[unshade]/np.log(10), color='b',
-                doline=doline, parprint=False, prob=.997, xlims=xlims[i])
-            tab.add_row([label, pltname[i], a1, a1_e, a0, a0_e, chi2, eps])
-        if pltname[i] == 'rdv':
-            a1, a1_e, a0, a0_e, chi2, eps = linefitting( np.log10(x[postive]), 
-                np.log10(y[postive]), xerr=xerr[postive]/np.log(10), 
-                yerr=yerr[postive]/np.log(10), color='b',
-                doline=False, parprint=False)
-            tab.add_row([label, pltname[i]+'all', a1, a1_e, a0, a0_e, chi2, eps])
+        if len(x[dofit]) > 2:
+            a1, a1_e, a0, a0_e, chi2, eps = linefitting( np.log10(x[dofit]), 
+                np.log10(y[dofit]), xerr=xerr[dofit]/np.log(10), 
+                yerr=yerr[dofit]/np.log(10), color='b',
+                doline=drawline, parprint=parprint, prob=.997, xlims=this_xlims)
+            tab.add_row([label, pltname[i], len(x[dofit]), a1, a1_e, a0, a0_e, chi2, eps])
+# Old code that would do a fit including unresolved points
+#         if pltname[i] == 'rdv':
+#             a1, a1_e, a0, a0_e, chi2, eps = linefitting( np.log10(x[dodisp]), 
+#                 np.log10(y[dodisp]), xerr=xerr[dodisp]/np.log(10), 
+#                 yerr=yerr[dodisp]/np.log(10), color='b',
+#                 doline=False, parprint=False)
+#             tab.add_row([label, pltname[i]+'all', len(x[dodisp]), a1, a1_e, a0, a0_e, chi2, eps])
         # Plot the binned values if nbin > 0
         if nbin > 0:
-            ymean, xbinedge, _ = stats.binned_statistic(np.log10(x[postive]), 
-                np.log10(y[postive]), statistic='mean', bins=nbin, range=xlims[i])
-            ystd, xbinedge, _  = stats.binned_statistic(np.log10(x[postive]), 
-                np.log10(y[postive]), statistic='std', bins=nbin, range=xlims[i])
+            ycnt, xbinedge, _ = stats.binned_statistic(np.log10(x[dodisp]),
+                np.log10(y[dodisp]), statistic='count', bins=nbin, range=this_xlims)
+            ymean, xbinedge, _ = stats.binned_statistic(np.log10(x[dodisp]), 
+                np.log10(y[dodisp]), statistic='mean', bins=nbin, range=this_xlims)
+            ystd, xbinedge, _  = stats.binned_statistic(np.log10(x[dodisp]), 
+                np.log10(y[dodisp]), statistic='std', bins=nbin, range=this_xlims)
             xbin = 0.5*(xbinedge[1:]+xbinedge[:-1])
-            plt.errorbar(xbin, ymean, yerr=ystd, ecolor='orange', marker='o', 
-                         ls='', mfc='yellow', mec='orange', zorder=10)
+            plt.errorbar(xbin[ycnt>1], ymean[ycnt>1], yerr=ystd[ycnt>1], ecolor='orange',
+                         marker='o', ls='', mfc='yellow', mec='orange', zorder=10)
         # Make the labels and draw the gray shaded boxes
-        std_overlay(pcat, [xplot[i], yplot[i]], xlims[i], ylims[i], shade)
+        std_overlay(pcat, [xplot[i], yplot[i]], this_xlims, this_ylims, shade)
+        if subpanel is not None:
+            axes.text(0.05,0.93, subpanel, size=12, transform=axes.transAxes)
         plt.legend(loc='lower right',fontsize='small',scatterpoints=1)
         plt.savefig(join(plotdir,label+'_'+pltname[i]+'_full.pdf'), bbox_inches='tight')
-        plt.close()
+        if noshow:
+            plt.close()
 
-        # --- Plot trunks or clusters and their descendants
-        clusttype = ['trunk', 'cluster']
-        for ilist, pltlist in enumerate([idsel[0], idsel[3]]):
-            if len(pltlist) > 0:
-                print('Plotting {} {}(s)'.format(len(pltlist),clusttype[ilist]))
-                fig, axes = plt.subplots(figsize=(6.4,4.8))
-                if xplot[i] == 'rad_pc' and yplot[i].startswith('m'):
-                    axes.set_aspect(0.6)
-                else:
-                    axes.set_aspect('equal')
-                sctplot( np.log10(x[pltlist]), np.log10(y[pltlist]), 
-                         xerr=xerr[pltlist]/np.log(10), yerr=yerr[pltlist]/np.log(10), 
-                         marker='s', zorder=4, cmap=cmap, ms=30)
-                colors = plt.cm.jet(np.linspace(0, 1, len(pltlist)) )
-                n_descend = 0
-                for j, tno in enumerate(pltlist):
-                    if ilist == 0:
-                        descdnts = [val for val in trd[j] if val in np.where(postive)[0]]
-                    else:
-                        descdnts = [val for val in cld[j] if val in np.where(postive)[0]]
-                    n_descend += len(descdnts)
-                    sctplot( np.log10(x[descdnts]), np.log10(y[descdnts]), 
-                             xerr=xerr[descdnts]/np.log(10), yerr=yerr[descdnts]/np.log(10), 
-                             col='w', mec=colors[j], zorder=3, ms=10, 
-                             label=clusttype[ilist]+str(tno) )
-                print('Plotting {} {} descendants'.format(n_descend,clusttype[ilist]))
-                std_overlay(pcat, [xplot[i], yplot[i]], xlims[i], ylims[i], shade)
-                # Only show legend if there are 10 or fewer trunks
-                if len(idsel[0]) <= 10:
-                    plt.legend(loc='lower right',fontsize='x-small',scatterpoints=1)
-                plt.savefig(join(plotdir,label+'_'+pltname[i]+'_'+clusttype[ilist]+'s.pdf'), 
-                            bbox_inches='tight')
+        # --- Plot trunks and their descendants
+        if len(idsel[0]) > 0:
+            print('Plotting {} {}(s)'.format(len(idsel[0]),'trunk'))
+            fig, axes = plt.subplots(figsize=(6.4,4.8))
+            if xplot[i] == 'rad_pc' and yplot[i].startswith('m'):
+                axes.set_aspect(0.6)
+            else:
+                axes.set_aspect('equal')
+            sctplot( np.log10(x[idsel[0]]), np.log10(y[idsel[0]]), 
+                     xerr=xerr[idsel[0]]/np.log(10), yerr=yerr[idsel[0]]/np.log(10), 
+                     marker='s', zorder=4, cmap=cmap, ms=30)
+            colors = plt.cm.jet(np.linspace(0, 1, len(idsel[0])) )
+            n_descend = 0
+            for j, tno in enumerate(idsel[0]):
+                descdnts = [val for val in trd[j] if val in np.where(dodisp)[0]]
+                n_descend += len(descdnts)
+                sctplot( np.log10(x[descdnts]), np.log10(y[descdnts]), 
+                         xerr=xerr[descdnts]/np.log(10), yerr=yerr[descdnts]/np.log(10), 
+                         col='w', mec=colors[j], zorder=3, ms=10, 
+                         label='trunk'+str(tno) )
+            print('Plotting {} {} descendants'.format(n_descend,'trunk'))
+            std_overlay(pcat, [xplot[i], yplot[i]], this_xlims, this_ylims, shade)
+            # Only show legend if there are 10 or fewer trunks
+            if len(idsel[0]) <= 10:
+                plt.legend(loc='lower right',fontsize='x-small',scatterpoints=1)
+            if subpanel is not None:
+                axes.text(0.05,0.93, subpanel, size=12, transform=axes.transAxes)
+            plt.savefig(join(plotdir,label+'_'+pltname[i]+'_trunks.pdf'), 
+                        bbox_inches='tight')
+            if noshow:
                 plt.close()
 
+        # --- Plot clusters
+        if len(idsel[3]) > 0:
+            print('Plotting {} {}(s)'.format(len(idsel[3]),'cluster'))
+            fig, axes = plt.subplots(figsize=(6.4,4.8))
+            if xplot[i] == 'rad_pc' and yplot[i].startswith('m'):
+                axes.set_aspect(0.6)
+            else:
+                axes.set_aspect('equal')
+            sctplot( np.log10(x[idsel[3]]), np.log10(y[idsel[3]]), col=clco,
+                     xerr=xerr[idsel[3]]/np.log(10), yerr=yerr[idsel[3]]/np.log(10), 
+                     marker='s', zorder=4, ms=30)
+            if len(x[idfit[3]]) > 2:
+                a1, a1_e, a0, a0_e, chi2, eps = linefitting( np.log10(x[idfit[3]]), 
+                    np.log10(y[idfit[3]]), xerr=xerr[idfit[3]]/np.log(10), 
+                    yerr=yerr[idfit[3]]/np.log(10), color='b',
+                    doline=drawline, parprint=parprint, prob=.997, xlims=this_xlims)
+                tab.add_row([label, pltname[i]+'_clust', len(x[idfit[3]]), a1, a1_e, 
+                            a0, a0_e, chi2, eps])
+            std_overlay(pcat, [xplot[i], yplot[i]], this_xlims, this_ylims, shade)
+            if subpanel is not None:
+                axes.text(0.05,0.93, subpanel, size=12, transform=axes.transAxes)
+            plt.savefig(join(plotdir,label+'_'+pltname[i]+'_clusters.pdf'), 
+                        bbox_inches='tight')
+            if noshow:
+                plt.close()
+
+        # ------ Plot all structures, color coded by 3rd variable and binned in quartiles
+        if not this_ccode:
+            continue
+        for j in range(len(colorcodes)):
+            fig, axes = plt.subplots(figsize=(6.4,4.8))
+            axes.set_aspect('equal')
+            print('Relation will be color coded by',colorcodes[j])
+            if colorcodes[j] not in cat.keys() and colorcodes[j] not in pcat.keys():
+                continue
+            if colorcodes[j] in cat.keys():
+                zcode = cat[colorcodes[j]][dodisp]
+                zlog = False
+            elif colorcodes[j] in ['axratio', 'refdist']:
+                zcode = pcat[colorcodes[j]][dodisp]
+                zlog = False
+            else:
+                zcode = np.log10(pcat[colorcodes[j]][dodisp])
+                zlog = True
+            color_code_bin(np.log10(x[dodisp]), np.log10(y[dodisp]), zcode, 
+                           colname=colorcodes[j], alpha=alpha, cmap=cmap, axes=axes,
+                           lobin_col=lobin_col, hibin_col=hibin_col, xlims=this_xlims, 
+                           zlog=zlog, nbin=nbin)
+            std_overlay(pcat, [xplot[i], yplot[i]], this_xlims, this_ylims, shade)
+            shortname = re.sub('_', '', colorcodes[j])
+            plt.savefig(join(plotdir,label+'_'+pltname[i]+'_'+shortname+'.pdf'), 
+                        bbox_inches='tight')
+            if noshow:
+                plt.close()
+
+    # ------ Write output files
     tab.write(join(indir, label+'_lfit.tex'), overwrite=True)    
+
+    if resolve_output != 'none':
+        if resolve_output == 'raw':
+            mask = ((pcat['rad_pc'] > shade['rad_pc']) & 
+                    (pcat['vrms_k'] > shade['vrms_k']) & 
+                    (pcat['area_pc2'] > shade['area_pc2']))
+        elif resolve_output == 'decon':
+            mask = (pcat['rad_pc_dcon'] > 0) & (pcat['vrms_k'] > 0)
+        print('Outputting {} out of {} dendros above resolution limits'.format(
+              len(pcat[mask]), len(pcat)))
+        pcat_out = catalog.replace('_full_catalog.txt','_physprop_resolve.txt')
+        pcat[mask].write(pcat_out, format='ascii.ecsv', overwrite=True)
+        cat_out = catalog.replace('_full_catalog.txt','_cat_resolve.txt')
+        cat[mask].write(cat_out, format='ascii.ecsv', overwrite=True)
+
     return
 
